# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    HumanStatus,
)


HUMAN_TRANSIENT_STATUSES: List[HumanStatus] = [
    HumanStatus.RUNNING,
]
"""
Lists transient statutes of the enum :class:`HumanStatus <HumanStatus>`.
"""
