# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    CockpitStatus,
)


COCKPIT_TRANSIENT_STATUSES: List[CockpitStatus] = [
    CockpitStatus.CREATING,
    CockpitStatus.DELETING,
    CockpitStatus.UPDATING,
]
"""
Lists transient statutes of the enum :class:`CockpitStatus <CockpitStatus>`.
"""
